{**************************************************
   splitmain.pas -- Model SDI application with split panes
   Version 1.2

   This is the main form of a the split panes SDI
   application. For a complete description and
   details, please see the howtouse.hlp file
   and look at the following help topics:
     "How to use this product"
     "Splitapp.dpr - An application template with split panes"

   Copyright (C) 1999-2001, Sanjay Kanade
   http://kanadepro.com/delphistuff/
**************************************************}

unit splitmain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Grids, Outline, FileCtrl,  
  Buttons, Menus, clipBrd;

type
  TMainWin = class(TForm)
    Status: TPanel;
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    ViewMenu: TMenuItem;
    HelpMenu: TMenuItem;
    FileExitItem: TMenuItem;
    HelpHelpTopicsItem: TMenuItem;
    bar11: TMenuItem;
    HelpAboutItem: TMenuItem;
    EditMenu: TMenuItem;
    EditCopyItem: TMenuItem;
    StatusMsg: TStaticText;
    HelpIntrodItem: TMenuItem;
    ViewOptionsItem: TMenuItem;
    leftPanel: TPanel;
    Splitter: TSplitter;
    rightPanel: TPanel;
    tempNote: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure HelpAboutItemClick(Sender: TObject);
    procedure FileExitItemClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure HelpHelpTopicsItemClick(Sender: TObject);
    procedure SplitterMoved(Sender: TObject);
    procedure HelpIntrodItemClick(Sender: TObject);
  private
    { Private declarations }

    procedure readRegistry(var isMax:boolean);
    procedure writeRegistry;
    procedure DoIdle(Sender: TObject; var Done: Boolean);
    procedure HintHandler(Sender: TObject);
    function DoHelp(Command: Word; Data: Longint;
           var CallHelp: Boolean): Boolean;
  public
    { Public declarations }
end;

var
  MainWin: TMainWin;

implementation

uses about, registry, commctrl, shellapi, commdefs,
     sskutils, reminder;

{$R *.DFM}

//*******************************************
procedure TMainWin.readRegistry(var isMax:boolean);
var
  reg: TRegistry;
begin
  reg := TRegistry.Create;
  try
    reg.OpenKey(PrmRegKeyBase, true);
    if PrmAlsoRestoreWindowPosition then
    begin
      left := reg.ReadInteger(RKeyX);
      top := reg.ReadInteger(RKeyY);
    end;
    Width := reg.ReadInteger(RKeyW);
    Height := reg.ReadInteger(RKeyH);
    leftPanel.width := reg.ReadInteger(RKeySplitAt);
    isMax := reg.ReadBool(RKeyIsMax);
  except
    //Put any defaults values here or before
    //the call to ReadRegistry in the caller function
  end;
  reg.Free;
end;

//*******************************************
procedure TMainWin.FormCreate(Sender: TObject);
var
  isMax: boolean;
begin
  isMax := false;

  //Remove this temporary message and memo in your own
  //applications. We didn't enter this message in
  //Object Inspector because it doesn't preserve the
  //paragraph.
  tempnote.text :=
    'Note: This sample application has all the features of the basic '+
    'application. In addition, it demonstrates how to remember the '+
    'position of the splitter in the registry. Try resizing the splitter '+
    'and stop and restart the application to verify that it remembers '+
    'the splitter position. For more details on the features of the basic application, '+
    'please click on the menu Help--Introudction.'#13#10#13#10+
     'When using this template as a base for your own applications, please '+
     'remove this note (a memo component).';

  StatusMsg.Caption := ' ';
  Application.OnIdle := DoIdle;
  Application.OnHelp := DoHelp;
  Application.OnHint := HintHandler;
  Application.Title := PrmProgName;
  Caption := PrmProgName;
  splitRatio := 0;
  //Setting the full path ensures that the help file will
  //be found even if your program changes directory through
  //a file open dialog
  application.helpFile := ExtractFilePath(Application.ExeName)
                      + PrmHelpFileName;
                      
  readRegistry(isMax);
  if isMax then
  begin
    //step down splitter width accordingly
    splitRatio := leftPanel.width/(screen.Width);
    leftPanel.width := Round(splitRatio * screen.Width);
    if (leftPanel.width < Splitter.MinSize) then
      leftPanel.Width := Splitter.MinSize;
    splitter.left := leftPanel.width;
  end
  else
  begin
    splitter.left := leftPanel.width;
    SplitterMoved(Self);
  end;

  //Instead of directly using help context numbers in the
  //object inspector, it is better to use the constants
  //from a header file generated by your help authoring tool.
  //My tool HelpHikes Pro can generate such a file. See
  //   http://web.superb.net/helphikes/
  //This constant comes from the file helpids.pas. Please
  //see the comments in that file.
  ViewOptionsItem.HelpContext := IDH_CHANGING_THE_OPTIONS;

  if isMax then
    windowState := wsMaximized;
end;

//*******************************************
procedure TMainWin.DoIdle(Sender: TObject; var Done: Boolean);
begin
  Application.OnIdle := nil;
  //If you want to show some startup message or screen, this
  //is the proper place to show it so that it will appear
  //after the main window appears. If you show it in the
  //FormCreate, it will appear before the main window is
  //shown and may not look nice. An example is an evaluation
  //reminder screen. It should appear on top of the main window
  //as a dialog to look nice.
  //Here is an example which shows a reminder box.
  remindAtStartup;
  
  splitRatio := Splitter.Left/(Self.ClientWidth);
end;

//*******************************************
procedure TMainWin.FormResize(Sender: TObject);
var
  newWidth: integer;
begin
  //This code is needed so that the splitter remains at the
  //same proportional position
  if (splitRatio <> 0) then
  begin
    newWidth := Round(splitRatio * Self.ClientWidth);
    if (newWidth < Splitter.MinSize) then
      leftPanel.Width := Splitter.MinSize
    else
      leftPanel.Width := NewWidth;
    splitter.left := leftPanel.width;
    SplitterMoved(Self);
  end;
end;

//*******************************************
procedure TMainWin.SplitterMoved(Sender: TObject);
begin
  splitRatio := Splitter.Left/(Self.ClientWidth);
end;

//*******************************************
procedure TMainWin.HelpAboutItemClick(Sender: TObject);
begin
  //This is the standard code to show a modal
  //dialog such as the About Box. When you define
  //such dialogs, please remember to remove them
  //from the Auto-Create forms box of the Project
  //Options--Forms dialog.
  with TAboutBox.Create(Self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

//*******************************************
procedure TMainWin.FileExitItemClick(Sender: TObject);
begin
  Close;
end;

//*******************************************
procedure TMainWin.writeRegistry;
var
  reg: TRegistry;
  windata: TWindowPlacement;
begin
    reg := TRegistry.Create;
    try
      reg.OpenKey(PrmRegKeyBase, true);
      windata.length := sizeof(windata);
      GetWindowPlacement(handle, @windata);
      //Note that we write X, Y in registry but don't use them unless ours
      //is a single instance application. See readRegistry.
      reg.WriteInteger(RKeyY, windata.rcNormalPosition.Top);
      reg.WriteInteger(RKeyX, windata.rcNormalPosition.Left);
      //Note that we can't write width and height for the following values.
      //They don't reflect the correct size in maximized position.
      reg.WriteInteger(RKeyW, windata.rcNormalPosition.right-windata.rcNormalPosition.left);
      reg.WriteInteger(RKeyH, windata.rcNormalPosition.bottom-windata.rcNormalPosition.top);
      reg.WriteInteger(RKeySplitAt, leftPanel.width);
      reg.WriteBool(RKeyIsMax, windowState=wsMaximized);
    finally
      reg.Free;
    end;
end;

//*******************************************
procedure TMainWin.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  //Let's store the properties in the registry
  //restore them on form create
  writeRegistry;
  //If you have any temporary files to delete or create external
  //files to keep state information, this is the right place to
  //do it.
end;

//*******************************************
//This is the code which shows the hints for the menus
//on the status bar
procedure TMainWin.HintHandler(Sender: TObject);
begin
  StatusMsg.Caption := Application.Hint;
  if (StatusMsg.Caption = '') then
    statusMsg.caption := ' ';
end;

//*******************************************
procedure TMainWin.HelpHelpTopicsItemClick(Sender: TObject);
begin
  application.HelpCommand(HELP_FINDER,0);
end;

//*******************************************
function TMainWin.DoHelp(Command: Word; Data: Longint;
  var CallHelp: Boolean): Boolean;
begin
  //This code avoids errors if help context numbers
  //are not defined for obvious menu items and F1 is
  //pressed for them. Otherwise, if you go the standard
  //way, it will produce an error message for F1 help
  //on a menu item for which there is no context number
  //defined. This also shows a nice way to direct the
  //context sensitive help to a secondary window.

  //Explanation of the code:
  //If you have specified a secondary window name (params.pas)
  //for all context sensitive help, it is used. All other
  //help calls are passed on to the VCL's help call.
  if (Command = HELP_CONTEXT) then
  begin
    CallHelp := False;
    if data <> 0 then
      //If you want to force some of the topics
      //to the main or another window, you can
      //use your conditional logic here. For
      //example, a topic describing a dialog might
      //not be appropriate for procwin so you can
      //send it to the main window
      //Example: if data = IDH_OPTIONS_DIALOG then
      //            showHelpInWindow(data, 'main')
      //         else...
      showHelpInWindow(Data, PrmHelpProcWin)
    else
    begin
      result := false;
      exit;
    end;
  end;
  Result := true;
end;

//*******************************************
procedure TMainWin.HelpIntrodItemClick(Sender: TObject);
begin
  //EXAMPLES of directly calling help:

  //Example 1:
  //Here is an example of displaying this topic in the main window.
  //I could have passed 'main' as the second parameter.
  showHelpInWindow(IDH_INTRODUCTION, '');

  //Here are other examples which you can test by uncommenting them
  //one by one and commenting the above Example 1 call so that only
  //one call exists.

  //Example 2:
  //Example of displaying this topic in a secondary window called 'procwin'
  //Note that this secondary window must be defined in your help file.
  //If you don't know what a secondary window is, please see my help
  //authoring tool HelpHikes Pro.
  //showHelpInWindow(IDH_INTRODUCTION, 'procwin');

  //Example 3:
  //Example of displaying this topic in a help popup window
  //showHelpInPopup(IDH_INTRODUCTION);

  //Notes on Delphi provided help related methods:
  //
  //HelpContext method:
  //You might ask, what if I want to use the HelpContext method of the
  //application object. If you use that method, it will go either to the
  //main window or to the secondary window, if you have defined one in
  //params.pas. Hence, you can use this call whenever you want the default
  //application behavior that you have set up for context sensitive help.
  //
  //HelpJump method:
  //If you want to use context strings instead of numbers, you can use
  //the helpJump method of the application object. However, this call
  //will always display the topic in the main window. How do you get
  //the context strings! Look at the file helpids.pas. The constant
  //names themselves are the context strings. Also, the topic names
  //appear as comments. Of course, how easy or difficult it is to
  //find the context strings depends on your help authoring tool. The
  //file helpids.pas is generated by my own help authoring tool
  //HelpHikes Pro. It stands as a single source for context strings
  //and context numbers for all the topics.
end;

end.
